// PointVector.cpp: implementation of the PointVector class.
//
//////////////////////////////////////////////////////////////////////

#include "PointVector.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PointVector::PointVector(float x,float y,float z)
{
	xvalue = x;
	yvalue = y;
	zvalue = z;
}

PointVector::~PointVector()
{

}

PointVector PointVector::CrossProduct(PointVector v2)
{
	// calculates cross product of v1 and v2 
	float returnx = (yvalue * v2.GetZ()) - (zvalue * v2.GetY());
	float returny = (zvalue * v2.GetX()) - (xvalue * v2.GetZ());
	float returnz = (xvalue * v2.GetY()) - (yvalue * v2.GetX());
	PointVector returnVector(returnx,returny,returnz);
	return returnVector;
}

void PointVector::SetX(float x)
{
	xvalue = x;
}

void PointVector::SetY(float y)
{
	yvalue = y;
}

void PointVector::SetZ(float z)
{
	zvalue = z;
}

float PointVector::GetX()
{
	return xvalue;
}

float PointVector::GetY()
{
	return yvalue;
}

float PointVector::GetZ()
{
	return zvalue;
}

float PointVector::DotProduct(PointVector v2)
{
	float result = xvalue * v2.GetX() + yvalue * v2.GetY() + zvalue * v2.GetZ();
	return result;
}

PointVector PointVector::CreateVector(PointVector p2)
{
	float returnx = p2.GetX() - xvalue;
	float returny = p2.GetY() - yvalue;
	float returnz = p2.GetZ() - zvalue;
	PointVector returnVector(returnx,returny,returnz);
	return returnVector;
}

void PointVector::Normalize()
{
	float magnitude = sqrt(	xvalue * xvalue + yvalue * yvalue + zvalue * zvalue );
	float finalx = xvalue / magnitude;
	float finaly = yvalue / magnitude;
	float finalz = zvalue / magnitude;
	xvalue = finalx;
	yvalue = finaly;
	zvalue = finalz;
}

PointVector::PointVector()
{
	xvalue = 0.0;
	yvalue = 0.0;
	zvalue = 0.0;
}

PointVector::PointVector(const PointVector &pv)
{
	xvalue = pv.xvalue;
	yvalue = pv.yvalue;
	zvalue = pv.zvalue;
}

PointVector PointVector::NormalizedCrossProduct(PointVector p2)
{
	PointVector returnVector = CrossProduct(p2);
	returnVector.Normalize();
	return returnVector;
}

void PointVector::MultiplyMatrix(float *matrix)
{
	float newx = xvalue*matrix[0] + yvalue*matrix[1] + zvalue*matrix[2];
	float newy = xvalue*matrix[4] + yvalue*matrix[5] + zvalue*matrix[6];
	float newz = xvalue*matrix[8] + yvalue*matrix[9] + zvalue*matrix[10];
	xvalue = newx;
	yvalue = newy;
	zvalue = newz;
}

const PointVector& PointVector::operator=(const PointVector& somepv)
{
	if(&somepv != this)
	{
		xvalue = somepv.xvalue;
		yvalue = somepv.yvalue;
		zvalue = somepv.zvalue;
	}
	return *this;
}

bool PointVector::operator==(const PointVector& somepv)
{
	return Equals(somepv);
}

bool PointVector::Equals(PointVector other)
{
	if((xvalue == other.xvalue) && (yvalue == other.yvalue) && (zvalue == other.zvalue))
		return true;
	else
		return false;
}

void PointVector::MultiplyScalar(float scalar)
{
	xvalue = xvalue * scalar;
	yvalue = yvalue * scalar;
	zvalue = zvalue * scalar;
}

PointVector PointVector::SubtractVector(PointVector vectorToSubtract)
{
	PointVector returnVector;
	returnVector.xvalue = xvalue - vectorToSubtract.xvalue;
	returnVector.yvalue = yvalue - vectorToSubtract.yvalue;
	returnVector.zvalue = zvalue - vectorToSubtract.zvalue;
	return returnVector;
}

void PointVector::SetXYZ(float x, float y, float z)
{
	xvalue = x;
	yvalue = y;
	zvalue = z;
}

void PointVector::Add(float xadd, float yadd, float zadd)
{
	xvalue += xadd;
	yvalue += yadd;
	zvalue += zadd;
}

void PointVector::SetXYZ(PointVector xyzPosition)
{
	xvalue = xyzPosition.GetX();
	yvalue = xyzPosition.GetY();
	zvalue = xyzPosition.GetZ();
}

float PointVector::DistanceBetweenPoint(PointVector p2)
{
	float xdifference = xvalue - p2.GetX();
	float ydifference = yvalue - p2.GetY();
	float zdifference = zvalue - p2.GetZ();

	float distance = sqrt(	xdifference * xdifference +
							ydifference * ydifference +
							zdifference * zdifference	);
	return distance;
}

void PointVector::Print()
{
	printf("PointVector: [%f,%f,%f]\n",xvalue,yvalue,zvalue);
}

void PointVector::Reverse()
{
	xvalue = -xvalue;
	yvalue = -yvalue;
	zvalue = -zvalue;
}

float PointVector::Length()
{
	float length = sqrt(xvalue * xvalue + yvalue * yvalue + zvalue * zvalue);
	return length;
}

void PointVector::Reset()
{
	xvalue = 0.0;
	yvalue = 0.0;
	zvalue = 0.0;
}

bool PointVector::XIsZero()
{
	if(xvalue == 0.0)
		return true;
	else
		return false;
}

bool PointVector::ZIsZero()
{
	if(zvalue == 0.0)
		return true;
	else
		return false;
}

PointVector PointVector::ProjectToXZ()
{
	PointVector returnVector;
	returnVector.SetX(xvalue);
	returnVector.SetY(0.0);
	returnVector.SetZ(zvalue);
	return returnVector;
}

PointVector PointVector::Add(PointVector other)
{
	PointVector returnVector;
	float newx = xvalue + other.xvalue;
	float newy = yvalue + other.yvalue;
	float newz = zvalue + other.zvalue;
	returnVector.SetX(newx);
	returnVector.SetY(newy);
	returnVector.SetZ(newz);
	return returnVector;
}
