#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "PointVector.h"
#include "WorldObject.h"

#define IGLOO 0
#define SNOW 1
#define WINTER 2
#define ICE 3
#define FIRE 4
#define NONE -1

#define ENEMIES 0
#define SNOWBALLS 1

#define PLAYERMODE 0
#define FREEFLYMODE 1

#define RENDER 0
#define SELECT 1
#define RENDER 0					// render mode integer macro
#define SELECT 1					// select mode integer macro
#define BUFSIZE 1024				// size of the picking buffer
// Struct for  position used to compare previous and current
// mouse coordinates
struct mouse{
  int x;
  int y;
  int differenceX;
  int differenceY;
  bool leftMouseDown;
  bool rightMouseDown;
  bool middleMouseDown;
  bool pitchPressed;
  bool yawPressed;
  bool rollPressed;
  bool movePressed;
  bool moved;
};

struct cameraMoveAngleData{
	float pitchAngle;
	float yawAngle;
	float rollAngle;
	float speed;
};

struct quadricNode{
	WorldObject objectDescription;	// WorldObject descriptions for collision
	GLUquadricObj* q;				// pointer to store quadratic
	quadricNode* next;				// next quadricNode
	int texture1;					// main texture
	int texture2;					// environment map texture
	int groundcounter;				// how many times bounced off the ground
	int hitcounter;					// how many times have enemy been hit
	int id;
	int spherecounter;
};

struct pointNode{
	WorldObject objectDescription;	// WorldObject description for position and velocity information
	pointNode* next;		// next pointNode in list
};

/* general data for window */
int windowHeight;
int windowWidth;
float ratio;
/* mouse stuff */
mouse mouseCoords;			// mouse information all packed into one
/* data for the camera */
int cameraMode;				// player mode or free fly mode
bool cameraStop;
cameraMoveAngleData cameraData;		// camera information all packed into one
/* saved rotation matrices */
float yawRotateMatrix[16];				// stores rotation about arbitrary yaw axis
float pitchRotateMatrix[16];			// stores rotation about arbitrary pitch axis
float rollRotateMatrix[16];				// stores rotation about arbitrary roll axis

/* vectors for the camera */
PointVector cameraPosition;		// camera's current position
PointVector cameraViewVector;		// camera's view vector
PointVector cameraUpVector;		// camera's up vector
PointVector cameraSideVector;		// camera's side vector

/* world object */
WorldObject fallingBall;
WorldObject staticBall;

quadricNode* spherelist;		// snowball list
quadricNode* staticspherelist;		// stationary bouncing spheres
quadricNode* enemylist;			// list of bouncing monsters
pointNode* snowlist;			// list of falling snow

/* textures */
GLuint* texList;
unsigned char* image1;
unsigned char* image2;
unsigned char* image3;
unsigned char* image4;
unsigned char* image5;

/* throwing snow balls */
int snowballcounter;	// number of existing snowballs at this time
int snowballidcounter;
int maxsnowballcount;	// max number of snowballs
float throwstrength;	// how hard is snowball thrown
float snowballmass;	// mass of snowballs thrown
float snowballsize;	// size of snowballs thrown
int maxgroundbounce;	// maximum number of times the object can bounce off the ground
int maxspherebounce;
int snowballdetail;	// how smooth is the snowball
/* world data */
PointVector worldGravity;	// gravity vector for the entire world
PointVector snowGravity;	// gravity vector for snow flakes, smaller than world gravity to simulate lighter snow
float worldwidth;			// width of the world
float worldlength;			// length of the world
float worldheight;

bool removeAll;				// remove all snow, enemies, and snowballs

int score;
/* enemy data */
int numberenemies;
float enemyheight;
float enemysize;
float enemymass;
int enemydetail;
int enemylife;
/* falling snow data */
int maxsnowcount;	// maximum number of snow flakes
int snowcounter;	// current count of snow flakes
int tickvalue;		// tick value at which snowdrops are created
int tickcounter;	// current tick count between 0 and tickvalue
int snowdensity;	// density of snowfall
bool snowmelt;
/* fps */
/*
int frame;		// number of frames since last computed framerate
int time;		// current number of milliseconds
int timebase;	// numebr of miliseconds since last computed framerate
int fps;		// the computed fps
*/
bool fpstoggle;	// show fps on or off
/* picking stuff */
GLuint selectBuf[BUFSIZE];	// used to store hit data
int snowballpicked;					// number for the snowball picked
int mouseCursorX;
int mouseCursorY;
int hits;
int mode;
bool pushnames;

////////////////////////////////////////////////
/* picking */
void ProcessHits(GLint hits, GLuint buffer[], int sw);
void StopPicking();
void StartPicking();

void InitData();
void InitWorldData(int level);
void InitCameraData();
void InitSphereData();
void GenerateRandomEnemies(int number);
bool FindEnemyOverlap(quadricNode* list,PointVector position);
/* effects */
void InitFog();
void InitLighting();
void InitTextures();
void RenderBitmapCharacter(float x,float y,float z,char *string);
void DrawText();

/* drawing */
void DrawWorldBox(float width,float length,float height,bool wireframe);
void DrawSnowMan();

void DrawFloor(float width,float length);
quadricNode* AddToQuadricList(quadricNode* somelist,int listtype,WorldObject object,int tex1,int tex2);
pointNode* AddToPointList(pointNode* somelist,WorldObject object);
int SolveSnowTick(int density,PointVector gravityVector);
void GenerateRandomSnow(int number);
void InitSnowFallData();

void RotateCameraPitch(GLfloat* pitchRotation);
void RotateCameraRoll(GLfloat* rollRotation);
void RotateCameraYaw(GLfloat* yawRotation);
void MoveCamera(float speed);

void ChangeSize(int w, int h);
void NormalKeys(unsigned char key, int x, int y);
void SpecialKeys(int key, int x, int y);
void ActiveMouseMotion(int x,int y);
void Mouse(int button,int state,int x,int y);
void RenderScene(void);

void TraverseDraw(quadricNode* somelist,int listtype,int detail);
void TraverseDrawPoints(pointNode* somelist);
quadricNode* TraverseRemove(quadricNode* somelist,int listtype);
void TraverseUpdate(quadricNode* somelist);
WorldObject TraverseSphereCollisionRoutine(WorldObject someobject,quadricNode* somelist,int listtype,float elasticity);
/* utility */
unsigned char* loadPNM( const char *filename,
     unsigned int& newwidth, unsigned int &newheight,
     unsigned int &numComponents );

///////////
// Light Parameters
//static GLfloat LightAmb[] = {0.7f, 0.7f, 0.7f, 1.0f};	// Ambient Light
//static GLfloat LightDif[] = {1.0f, 1.0f, 1.0f, 1.0f};	// Diffuse Light
//static GLfloat LightPos[] = {4.0f, 4.0f, 6.0f, 1.0f};	// Light Position
static GLfloat LightAmb[] = {1.0,1.0,1.0,1.0f};	// Ambient Light
static GLfloat LightDif[] = {1.0,0.6,0.6,1.0};	// Diffuse Light
static GLfloat LightPos[] = {4.0f, worldheight, 6.0f, 1.0f};	// Light Position

void DrawObject();
int InitGL(GLvoid);
void Display(GLvoid);

